'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

const path = require('path');
const chalk = require('chalk');
const consola2 = _interopDefault(require('consola'));
const defu2 = _interopDefault(require('defu'));

const apiHost = "api.sanity.io";
const cdnHost = "apicdn.sanity.io";
const enc = encodeURIComponent;
function getQs(query, params = {}) {
  const baseQs = `?query=${enc(query)}`;
  return Object.keys(params).reduce((current, param) => {
    return `${current}&${enc(`$${param}`)}=${enc(JSON.stringify(params[param]))}`;
  }, baseQs);
}
function createClient(config) {
  const {projectId, dataset, useCdn, withCredentials, token} = config;
  return {
    clone: () => createClient({projectId, dataset, useCdn, withCredentials, token}),
    async fetch(query, params) {
      const host = useCdn ? cdnHost : apiHost;
      const qs = getQs(query, params);
      const response = await fetch(`https://${projectId}.${host}/v1/data/query/${dataset}${qs}`, {
        credentials: withCredentials ? "include" : "omit",
        headers: {
          ...token ? {
            Authorization: `Bearer ${token}`
          } : {},
          Accept: "application/json",
          ...process.server ? {"accept-encoding": "gzip, deflate"} : {}
        }
      });
      const {result} = await response.json();
      return result;
    }
  };
}

const groq = (strings, ...keys) => {
  const lastIndex = strings.length - 1;
  return strings.slice(0, lastIndex).reduce((query, currentString, index) => query + currentString + keys[index], "") + strings[lastIndex];
};

const isNuxtBuild = process.client || process.server;
const isProd = process.env.NODE_ENV === "production";
const DEFAULTS = {
  contentHelper: true,
  imageHelper: true,
  dataset: "production",
  withCredentials: false
};
const CONFIG_KEY = "sanity";
function validateConfig({projectId, dataset}) {
  if (!projectId) {
    consola2.warn(`Make sure you specify a ${chalk.bold("projectId")} in your sanity config.`);
    return false;
  } else {
    consola2.info(`Enabled ${chalk.bold("@nuxtjs/sanity")} for project ${chalk.bold(projectId)} (${chalk.bold(dataset)}).`);
    return true;
  }
}
const nuxtModule = function(moduleOptions) {
  if (isNuxtBuild) {
    return;
  }
  let sanityConfig = {};
  try {
    if (process.server) {
      const fs = require("fs-extra");
      const {projectId, dataset} = fs.readJSONSync(path.resolve(this.options.rootDir, "./sanity.json")).api;
      sanityConfig = {projectId, dataset};
    }
  } catch (e) {
  }
  const options = defu2(this.options[CONFIG_KEY], moduleOptions, sanityConfig, {useCdn: isProd && !moduleOptions.token && !this.options[CONFIG_KEY].token}, DEFAULTS);
  if (!validateConfig(options)) {
    return;
  }
  let useOfficialClient = !options.minimal;
  try {
    if (useOfficialClient) {
      require("@sanity/client");
    }
  } catch (e) {
    useOfficialClient = false;
    consola2.warn(`Not using ${chalk.bold("@sanity/client")} as it cannot be resolved in your project dependencies.
       Try running ${chalk.bold("yarn add @sanity/client")} or ${chalk.bold("npm install @sanity/client")}.
       To disable this warning, set ${chalk.bold("sanity: { minimal: true }")} in your nuxt.config.js.`);
  }
  this.addPlugin({
    src: path.resolve(__dirname, "../templates/plugin.js"),
    fileName: "sanity/plugin.js",
    options: {
      client: useOfficialClient,
      components: {
        imageHelper: options.imageHelper,
        contentHelper: options.contentHelper
      },
      sanityConfig: JSON.stringify({
        useCdn: options.useCdn,
        projectId: options.projectId,
        dataset: options.dataset,
        withCredentials: options.withCredentials,
        token: options.token
      })
    }
  });
  if (options.imageHelper) {
    this.addTemplate({
      src: path.resolve(__dirname, "../dist/sanity-image.js"),
      fileName: "sanity/sanity-image.js",
      options: {
        projectId: options.projectId,
        dataset: options.dataset
      }
    });
  }
  this.options.build.transpile = this.options.build.transpile || [];
  this.options.build.transpile.push(/^@nuxtjs[\\/]sanity/);
};
nuxtModule.meta = {name: "@nuxtjs/sanity"};

exports.createClient = createClient;
exports.default = nuxtModule;
exports.getQs = getQs;
exports.groq = groq;
